/*
 * file StpAccessControlEntry.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.StpAccessControlEntry
 *
 *  Copyright IBM Corporation 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.WvcmException;

import com.ibm.rational.wvcm.stp.cq.CqProvider;
import com.ibm.rational.wvcm.stp.cq.CqQueryFolder;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;
import com.ibm.rational.wvcm.stpex.StpExEnumerationBase;

/**
 * The interface for an entry in a resource's access control list. Each access
 * control entry denotes <i>access rights</i> to a specific resource granted to
 * one or more <i>principals</i> (denoted by a group, user, or role).
 * <p>
 * In addition, each StpAccessControlEntry instance returned by this API
 * specifies whether the entry represents an access right applied directly to
 * the resource or an access right derived from access rights applied to one or
 * more related resources. In most cases, the resource to which the effective
 * access right is actually applied is also included in the
 * StpAccessControlEntry returned from the API.
 * <p>
 * To add an access control entry to a resource an instance of
 * StpAccessControlEntry is required. It can be constructed using
 * {@link StpProvider#buildAccessControlEntry(StpPrincipal, com.ibm.rational.wvcm.stp.StpAccessControlEntry.AccessRight[])}
 * if the access right is known a priori or by requesting the
 * {@link CqQueryFolder#POSSIBLE_ACCESS_RIGHTS} property and selecting one of
 * those. When adding an access control entry to a resource, only the access
 * rights and principal fields of the StpAccessControlEntry object are
 * significant.
 * <p>
 * Resources that may have access control lists applied them implement the 
 * {@link StpAccessControlledResource} interface.
 * <p>
 * Resources that represent entities to which access rights may be granted 
 * implement the {@link StpPrincipal} interface.
 */
public interface StpAccessControlEntry
{
    /**
     * @return <b>true</b> iff this access control entry grants reading of the
     *         controlled resource to the principal
     */
    boolean allowsRead();

    /**
     * @return <b>true</b> iff this access control entry allows writing of the
     *         controlled resource by the principal
     */
    boolean allowsWrite();

    /**
     * @return The group, user, or role that is granted the access rights of
     *         this entry to the controlled resource; <b>null</b> if no
     *         principal has been associated with this permission.
     */
    StpPrincipal getPrincipal();

    /**
     * @return The access rights granted by this entry to the principal on the
     *         controlled resource. Will never be <b>null</b>.
     */
    AccessRight[] getAccessRights();

    /**
     * An enumeration of the possible access rights that can be applied to a
     * controlled resource. Not all resources will support all access rights.
     */
    public enum AccessRight implements StpExEnumeration
    {
        /** Principal has no access to the controlled resource */
        NO_ACCESS("no-access"),
        /** Principal can only read the controlled resource */
        READ_ONLY("read-only"),
        /** Principal can read and write the controlled resource */
        READ_WRITE("read-write"),
        /** Principal has limited read access to the controlled resource */
        READ_LIMITED("read-limited"),
        /** Principal can change access rights to the controlled resource */
        CHANGE_PERMISSION("change-permission");

        /** The name of the access right for display purposes */
        public String toString()
        {
            return m_name;
        }

        private AccessRight(String name)
        {
            m_name = name;
        }

        private String m_name;
    };

    /**
     * @return The resource whose access is controlled by this access control
     *         entry; will be null if no resource has been associated with this
     *         StpAccessControlEntry instance.
     */
    StpAccessControlledResource getControlledResource();

    /**
     * Assigns a new value for the principal of this access control entry.
     * 
     * @param principal An StpResource proxy specifying the group, user, or role
     *            identifying the principals granted access rights by this
     *            entry. Not all resources accept all types of principals.
     */
    void setPrincipal(StpPrincipal principal);

    /**
     * Defines a new value for the access rights granted by this entry.
     * 
     * @param accessRights An array of access right instances specifying the
     *            access rights granted by this entry; must not be <b>null</b>
     *            or empty.
     */
    void setAccessRights(AccessRight... accessRights);

    /**
     * Assigns a new value to the controlled resource of this entry.
     * 
     * @param controlledResource An StpResource proxy specifying the resource to
     *            be controlled by this entry. Not all resource types can be
     *            controlled by an access control list.
     */
    void setControlledResource(StpAccessControlledResource controlledResource);

    /**
     * An enumeration of the type of permission being reported
     */
    public enum EntryType implements StpExEnumeration
    {
        /**
         * An entry applied explicitly to the controlled resource.
         */
        APPLIED,

        /**
         * An effective entry derived from entries directly applied
         * to one or more resources associated with the controlled resources.
         */
        EFFECTIVE;
    };

    /**
     * @return The type of access control entry being reported by this instance.
     */
    EntryType getEntryType();

    /**
     * @return The resource to which this access control entry was actually
     *         applied, if known; <b>null</b> otherwise.
     */
    StpAccessControlledResource getAppliedResource();
}
